"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/* Copyright IBM Corp. 2017 */
var path_1 = require("path");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var rx_file_1 = require("../src/utils/rx.file");
var asset_dependencies_1 = require("./../src/utils/asset.dependencies");
var rx_delete_1 = require("./../src/utils/rx.delete");
var rx_walk_1 = require("./../src/utils/rx.walk");
var test_utils_1 = require("./utils/test.utils");
describe('asset.dependencies', function () {
    it('should copy dependencies in correct order', function () {
        var assetBase = test_utils_1.getAssetBaseDir();
        var pkgTest = path_1.join(assetBase, 'test', 'deps', 'package.json');
        var rxDstBase = rx_walk_1.rxTempDir().pipe(operators_1.shareReplay(1));
        var rxDelete = rxDstBase.pipe(operators_1.mergeMap(function (dstBase) { return rx_delete_1.rxEmptyDir(dstBase); }));
        var rxCopy = rxDstBase.pipe(operators_1.mergeMap(function (dstBase) { return asset_dependencies_1.rxCopyDependencies(pkgTest, dstBase); }));
        var rxIds = rxCopy.pipe(operators_1.mergeMap(rx_file_1.rxReadTextFile), operators_1.map(function (data) { return JSON.parse(data); }), operators_1.map(function (data) { return data.id; }), operators_1.reduce(function (map, key) {
            map[key] = key;
            return map;
        }, {}), operators_1.tap(function (res) {
            expect(res).toHaveProperty('asset1_root');
            expect(res).toHaveProperty('asset2_root');
            expect(res).toHaveProperty('asset3_dep1');
            expect(res).toHaveProperty('asset4_dep1');
            expect(res).toHaveProperty('asset5_dep2');
        }));
        var rxResult = rxjs_1.concat(rxDelete, rxIds, rxDelete);
        return rxResult.toPromise();
    });
    it('should resolve dependencies in correct order', function () {
        var assetBase = test_utils_1.getAssetBaseDir();
        var pkgTest = path_1.join(assetBase, 'test', 'deps', 'package.json');
        var depRoot = path_1.normalize(path_1.join(assetBase, 'test', 'deps', 'data')), dep1 = path_1.normalize(path_1.join(assetBase, 'test', 'deps', 'node_modules', 'dep1', 'data')), dep2 = path_1.normalize(path_1.join(assetBase, 'test', 'deps', 'node_modules', 'dep2', 'data'));
        var rxResult = asset_dependencies_1.rxResolveDependencies(pkgTest);
        return rxResult.pipe(operators_1.toArray(), operators_1.tap(function (array) {
            expect(array).toHaveLength(3);
            expect(array[0]).toBe(dep2);
            expect(array[1]).toBe(dep1);
            expect(array[2]).toBe(depRoot);
        })).toPromise();
    });
    it('should resolve dependencies in reverse order', function () {
        var assetBase = test_utils_1.getAssetBaseDir();
        var pkgTest = path_1.join(assetBase, 'test', 'deps', 'package.json');
        var depRoot = path_1.normalize(path_1.join(assetBase, 'test', 'deps', 'data')), dep1 = path_1.normalize(path_1.join(assetBase, 'test', 'deps', 'node_modules', 'dep1', 'data')), dep2 = path_1.normalize(path_1.join(assetBase, 'test', 'deps', 'node_modules', 'dep2', 'data'));
        var rxResult = asset_dependencies_1.rxResolveDependencies(pkgTest, true);
        return rxResult.pipe(operators_1.toArray(), operators_1.tap(function (array) {
            expect(array).toHaveLength(3);
            expect(array[2]).toBe(dep2);
            expect(array[1]).toBe(dep1);
            expect(array[0]).toBe(depRoot);
        }))
            .toPromise();
    });
});
