"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
/* Copyright IBM Corp. 2017 */
var path_1 = require("path");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var angular_utils_1 = require("../utils/angular.utils");
var file_utils_1 = require("../utils/file.utils");
var constants_1 = require("./../constants");
var config_1 = require("./../utils/config");
var rx_copy_1 = require("./../utils/rx.copy");
var rx_delete_1 = require("./../utils/rx.delete");
var rx_move_1 = require("./../utils/rx.move");
var abstract_command_1 = require("./abstract.command");
var help_1 = require("./help");
var ROOT_RESOURCE = {
    'index.html': true,
    'favicon.ico': true
};
function _isRootResource(aFilename) {
    var parsed = path_1.parse(aFilename);
    return !!ROOT_RESOURCE[parsed.base];
}
/**
 * Copies the build output to the correct location
*/
var CopyBuildCommand = /** @class */ (function (_super) {
    __extends(CopyBuildCommand, _super);
    function CopyBuildCommand(options) {
        return _super.call(this, options) || this;
    }
    CopyBuildCommand.prototype.printHelp = function () {
        // log the command line
        console.log(CopyBuildCommand.COMMAND, CopyBuildCommand.TYPE, "--" + constants_1.OPTION_SOURCE + " <SOURCE>", "--" + constants_1.OPTION_DATA + " <DATA>", "--" + constants_1.OPTION_NAME + " <NAME>", "[--" + constants_1.OPTION_ROOT + " <ROOT>]", "[--" + constants_1.OPTION_VERBOSE + "]");
        // log the parameters
        console.log("--" + constants_1.OPTION_SOURCE + " <SOURCE>", ':', help_1.HELP_COPY_SOURCE);
        console.log("--" + constants_1.OPTION_DATA + " <DATA>", ':', help_1.HELP_DATA_DIRECTORY);
        console.log("--" + constants_1.OPTION_NAME + " <NAME>", ':', help_1.HELP_APPLICATION_NAME);
        console.log("--" + constants_1.OPTION_ROOT, ':', help_1.HELP_CONTEXT_ROOT);
        console.log("--" + constants_1.OPTION_VERBOSE, ':', help_1.HELP_VERBOSE);
    };
    CopyBuildCommand.prototype.process = function () {
        // the options
        var opts = this.getOptions();
        var appName = opts.name || this.getAppName();
        var contextRoot = "/" + config_1.configTrimSlashes(opts.root || this.getContextRoot(appName));
        // logger
        var logger = this.getLogger();
        // the directories
        var dataDir = this.getDataDir();
        // source directory is the build output
        var firstSrc = file_utils_1.firstOf(opts.src);
        var rxSrcDir = firstSrc ? rxjs_1.of(path_1.resolve(firstSrc)) : this.getAngularCliConfig().pipe(operators_1.map(angular_utils_1.getDistDir), operators_1.map(function (dir) { return path_1.resolve(dir); }));
        // name of the target directory for the main files
        var assetDir = path_1.resolve(path_1.normalize(path_1.join(dataDir, 'assets', appName)));
        // directory for index file
        var idxDir = config_1.configIndexDirFromContextRoot(contextRoot, dataDir);
        // delete the files in the asset dir
        var rxDeletedAssets = rx_delete_1.rxDeleteDir(assetDir);
        // copy files into the assets directory
        var rxCopiedAssets = rxjs_1.combineLatest(rxSrcDir, rxDeletedAssets).pipe(operators_1.tap(function (_a) {
            var srcDir = _a[0], dstDir = _a[1];
            return logger.next("Copying from [" + srcDir + "] to [" + dstDir + "].");
        }), operators_1.mergeMap(function (_a) {
            var srcDir = _a[0], dstDir = _a[1];
            return rx_copy_1.rxCopyDir(srcDir, dstDir);
        }), operators_1.shareReplay());
        // copy some selected files to the context root dir
        var rxRoot = rxCopiedAssets.pipe(operators_1.filter(_isRootResource));
        var rxApp = rxCopiedAssets.pipe(operators_1.filter(function (res) { return !_isRootResource(res); }));
        // move the root resources
        var rxMovedRoot = rxRoot.pipe(operators_1.mergeMap(function (file) { return rx_move_1.rxMoveFile(file, path_1.join(idxDir, path_1.parse(file).base)); }));
        // merge
        var rxResult = rxjs_1.merge(rxApp, rxMovedRoot);
        return rxResult;
    };
    CopyBuildCommand.TYPE = 'build';
    CopyBuildCommand.COMMAND = constants_1.CMD_COPY;
    return CopyBuildCommand;
}(abstract_command_1.AbstractCommand));
exports.CopyBuildCommand = CopyBuildCommand;
