"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
/* Copyright IBM Corp. 2017 */
var graceful_fs_1 = require("graceful-fs");
var path_1 = require("path");
var rxjs_1 = require("rxjs");
var typescript_1 = require("typescript");
var file_utils_1 = require("../utils/file.utils");
var constants_1 = require("./../constants");
var typescript_utils_1 = require("./../utils/typescript.utils");
var abstract_command_1 = require("./abstract.command");
var KEY_LAYOUT_COMPONENT = 'LayoutComponent';
var KEY_COMPONENT = 'Component';
function _decodeLayout(aDecs, aClass) {
    var layout = aDecs[KEY_COMPONENT];
    if (typescript_1.isCallExpression(layout.expression)) {
        // get the arguments
        var args = layout.expression.arguments;
        if (args.length > 0) {
            // first arg is the array
        }
        console.log('layout', layout.expression.arguments);
        layout.expression.arguments.forEach(function (a) { return typescript_1.forEachChild(a, function (c) { return console.log('child', c); }); });
    }
    return null;
}
function _isLayoutComponent(aDecs) {
    return aDecs[KEY_LAYOUT_COMPONENT] && aDecs[KEY_COMPONENT];
}
/** visit nodes finding classes */
function _visit(node, checker) {
    if (typescript_1.isClassDeclaration(node)) {
        var cd = node;
        var decs = typescript_utils_1.tsGetDecorators(cd);
        if (_isLayoutComponent(decs)) {
            _decodeLayout(decs, cd);
        }
        else {
            // This is a top level class, get its symbol
            typescript_1.forEachChild(node, function (f) { return _visit(f, checker); });
        }
    }
    else if (typescript_1.isModuleDeclaration(node)) {
        // This is a namespace, visit its children
        typescript_1.forEachChild(node, function (f) { return _visit(f, checker); });
    }
}
function _getClasses(aProgram) {
    // the type checker
    var typeChecker = aProgram.getTypeChecker();
    // iterate
    // Visit every sourceFile in the program
    for (var _i = 0, _a = aProgram.getSourceFiles(); _i < _a.length; _i++) {
        var sourceFile = _a[_i];
        // Walk the tree to search for classes
        typescript_1.forEachChild(sourceFile, function (f) { return _visit(f, typeChecker); });
    }
}
function _readCompilerOptions(aPath) {
    return new Promise(function (aResolve, aReject) { return graceful_fs_1.readFile(aPath, 'utf-8', function (err, data) {
        if (err) {
            aReject(err);
        }
        else {
            // parse the options
            var opt = JSON.parse(data).compilerOptions;
            opt.configFilePath = aPath;
            opt.moduleResolution = typescript_1.ModuleResolutionKind.NodeJs;
            aResolve(opt);
        }
    }); });
}
var CreateMappingsCommand = /** @class */ (function (_super) {
    __extends(CreateMappingsCommand, _super);
    function CreateMappingsCommand(options) {
        var _this = _super.call(this, options) || this;
        _this.options = options;
        return _this;
    }
    CreateMappingsCommand.prototype.process = function () {
        var opt = this.options;
        // load the compiler options
        var rootDir = path_1.resolve(file_utils_1.firstOf(opt.src) || this.getAppDir());
        var tsconfig = path_1.join(rootDir, 'tsconfig.json');
        var cmpOptions = _readCompilerOptions(tsconfig);
        // build the program
        var program = cmpOptions
            .then(function (cmdOpts) { return typescript_1.createProgram([path_1.join(rootDir, cmdOpts.baseUrl || '', 'main.ts')], cmdOpts); });
        // the classes
        var classes = program.then(_getClasses);
        // process
        return rxjs_1.from(classes.then(function () { return ''; }));
    };
    CreateMappingsCommand.TYPE = 'mappings';
    CreateMappingsCommand.COMMAND = constants_1.CMD_CREATE;
    return CreateMappingsCommand;
}(abstract_command_1.AbstractCommand));
exports.CreateMappingsCommand = CreateMappingsCommand;
