#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var minimist = require("minimist");
var factory_1 = require("../commands/factory");
var constants_1 = require("../constants");
function runCli(argv) {
    var OPTIONS = {
        string: [constants_1.OPTION_TYPE, constants_1.OPTION_DATA, constants_1.OPTION_SOURCE, constants_1.OPTION_URL, constants_1.OPTION_NAME, constants_1.OPTION_INCLUDE_PATTERN, constants_1.OPTION_EXCLUDE_PATTERN,
            constants_1.OPTION_USER, constants_1.OPTION_PASSWORD, constants_1.OPTION_COUNT, constants_1.OPTION_MODULE, constants_1.OPTION_OUT, constants_1.OPTION_ROOT, constants_1.OPTION_ID, constants_1.OPTION_LAYOUT_HTML_TEMPLATE,
            constants_1.OPTION_PKG_DIR, constants_1.OPTION_ARCHIVE],
        boolean: [constants_1.OPTION_VERBOSE, constants_1.OPTION_SCSS, constants_1.OPTION_FLAT, constants_1.OPTION_OVERRIDE, constants_1.OPTION_SKIP_REGISTRATION,
            constants_1.OPTION_SKIP_PULL, constants_1.OPTION_AGGREGATED, constants_1.OPTION_SKIP_SOURCEMAPS,
            constants_1.OPTION_RELATIVE, constants_1.OPTION_SCRIPT, constants_1.OPTION_HELP, constants_1.OPTION_ESI, constants_1.OPTION_EDITABLE, constants_1.OPTION_MODULES],
        alias: {},
        default: {}
    };
    // defaults
    var defaults = OPTIONS.default || {};
    defaults[constants_1.OPTION_VERBOSE] = false;
    defaults[constants_1.OPTION_SCSS] = false;
    defaults[constants_1.OPTION_FLAT] = false;
    defaults[constants_1.OPTION_HELP] = false;
    defaults[constants_1.OPTION_OVERRIDE] = false;
    defaults[constants_1.OPTION_SKIP_REGISTRATION] = false;
    defaults[constants_1.OPTION_SKIP_SOURCEMAPS] = false;
    defaults[constants_1.OPTION_SKIP_PULL] = false;
    defaults[constants_1.OPTION_RELATIVE] = false;
    defaults[constants_1.OPTION_SCRIPT] = false;
    defaults[constants_1.OPTION_ESI] = false;
    defaults[constants_1.OPTION_MODULES] = false;
    defaults[constants_1.OPTION_EDITABLE] = false;
    defaults[constants_1.OPTION_AGGREGATED] = false;
    defaults[constants_1.OPTION_COUNT] = '1';
    // alias
    var alias = OPTIONS.alias || {};
    alias[constants_1.OPTION_COUNT] = [constants_1.OPTION_N];
    var args = minimist(argv.slice(2), OPTIONS), cmd = args['_'];
    var opts = {
        archive: args[constants_1.OPTION_ARCHIVE],
        data: args[constants_1.OPTION_DATA],
        src: args[constants_1.OPTION_SOURCE],
        url: args[constants_1.OPTION_URL],
        user: args[constants_1.OPTION_USER],
        password: args[constants_1.OPTION_PASSWORD],
        name: args[constants_1.OPTION_NAME],
        include: args[constants_1.OPTION_INCLUDE_PATTERN],
        exclude: args[constants_1.OPTION_EXCLUDE_PATTERN],
        scss: args[constants_1.OPTION_SCSS],
        flat: args[constants_1.OPTION_FLAT],
        override: args[constants_1.OPTION_OVERRIDE],
        type: args[constants_1.OPTION_TYPE],
        module: args[constants_1.OPTION_MODULE],
        modules: args[constants_1.OPTION_MODULES],
        skipRegistration: args[constants_1.OPTION_SKIP_REGISTRATION],
        skipSourcemaps: args[constants_1.OPTION_SKIP_SOURCEMAPS],
        skipPull: args[constants_1.OPTION_SKIP_PULL],
        count: Number.parseInt(args[constants_1.OPTION_COUNT] || '1'),
        relative: args[constants_1.OPTION_RELATIVE],
        script: args[constants_1.OPTION_SCRIPT],
        aggregated: args[constants_1.OPTION_AGGREGATED],
        command: cmd[0],
        commandType: cmd[1],
        out: args[constants_1.OPTION_OUT],
        root: args[constants_1.OPTION_ROOT],
        id: args[constants_1.OPTION_ID],
        verbose: args[constants_1.OPTION_VERBOSE],
        help: args[constants_1.OPTION_HELP],
        layoutHtmlTemplate: args[constants_1.OPTION_LAYOUT_HTML_TEMPLATE],
        esi: args[constants_1.OPTION_ESI],
        editable: args[constants_1.OPTION_EDITABLE],
        pkgDir: args[constants_1.OPTION_PKG_DIR],
        allArgs: argv.slice(2)
    };
    if (opts.verbose) {
        // log the options without the password
        var loggingOpts = Object.assign({}, opts);
        delete loggingOpts.password;
        // log
        console.log(loggingOpts);
    }
    var fct = new factory_1.CommandFactory();
    var command = fct.create(opts);
    // execute
    if (opts.help) {
        command.printHelp();
    }
    else {
        command.process().subscribe(console.log, function (error) {
            console.error(error);
            process.exit(1);
        });
    }
}
exports.runCli = runCli;
