"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var graceful_fs_1 = require("graceful-fs");
var path_1 = require("path");
var wchtools_1 = require("./wchtools");
function _readPackageConfig(aPackageDir) {
    // read the config
    var filename = path_1.join(aPackageDir, 'package.json');
    // read
    try {
        return JSON.parse(graceful_fs_1.readFileSync(filename, 'utf-8')).config || {};
    }
    catch (e) {
        return {};
    }
}
exports.configReadPackageConfig = _readPackageConfig;
function _getDataDir(aPackageDir, aConfig) {
    // check if we have a configured application name
    return (aConfig && aConfig.data) ? path_1.normalize(path_1.join(aPackageDir, aConfig.data)) : undefined;
}
exports.configGetDataDir = _getDataDir;
function _getDefaultDataDir(aPackageDir) {
    // check if we have a configured application name
    return path_1.normalize(path_1.join(aPackageDir, 'data'));
}
exports.configGetDefaultDataDir = _getDefaultDataDir;
/**
 * Returns the directory of the application, typcially './src'
 *
 * @param aPackageDir   package directory
 * @param aConfig       config directory
 */
function _getAppDir(aPackageDir, aConfig) {
    // check if we have a configured application name
    return (aConfig && aConfig.src) ? path_1.normalize(path_1.join(aPackageDir, aConfig.src)) : path_1.normalize(path_1.join(aPackageDir, 'src'));
}
exports.configGetAppDir = _getAppDir;
/**
 * Returns the directory of the application, typcially './src'
 *
 * @param aPackageDir   package directory
 * @param aConfig       config directory
 */
function _getDefaultAppDir(aPackageDir) {
    return path_1.normalize(path_1.join(aPackageDir, 'src'));
}
exports.configGetDefaultAppDir = _getDefaultAppDir;
function _getAppName(aPackageDir, aConfig) {
    // check if we have a configured application name
    return (aConfig && aConfig.name) ? aConfig.name : undefined;
}
exports.configGetAppName = _getAppName;
function _getContextRoot(aPackageDir, aConfig) {
    // check if we have a configured application name
    return (aConfig && aConfig.root) ? aConfig.root : undefined;
}
exports.configGetContextRoot = _getContextRoot;
// trim slashes at the beginning and the end
var TRIM_REGEXP = /^[\/]+|[\/]+$/g;
/**
 * Trims leading and trailing slashes
 *
 * @param aAppName  the app name
 * @return the trimmed slashes
 */
function _trimSlashes(aAppName) {
    return aAppName ? aAppName.replace(TRIM_REGEXP, '') : '';
}
exports.configTrimSlashes = _trimSlashes;
function _createUrl(aAppName, aApiBaseUrl) {
    // build
    var deliveryUrl = wchtools_1.wchDeliveryUrlFromApiUrl(aApiBaseUrl);
    var trimmedName = _trimSlashes(aAppName);
    // build the base URL
    return trimmedName ? "" + deliveryUrl + encodeURIComponent(trimmedName) + "/" : deliveryUrl;
}
exports.configGetBaseUrl = _createUrl;
function _getIndexDirFromContextRoot(aContextRoot, aDataDir) {
    // split the root into path elements
    return path_1.resolve(path_1.normalize(path_1.join.apply(void 0, [aDataDir, 'assets'].concat(aContextRoot.split('/').map(decodeURIComponent)))));
}
exports.configIndexDirFromContextRoot = _getIndexDirFromContextRoot;
