"use strict";
exports.__esModule = true;
/* Copyright IBM Corp. 2017 */
var path = require("path");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var path_1 = require("path");
var config_1 = require("./../utils/config");
var rx_walk_1 = require("./../utils/rx.walk");
var transliteration_1 = require("./../utils/transliteration");
var wchtools_1 = require("./../utils/wchtools");
var rx_file_1 = require("../utils/rx.file");
var AbstractCommand = /** @class */ (function () {
    function AbstractCommand(abstractOptions) {
        this.abstractOptions = abstractOptions;
        this.resourceRoot = path.resolve(__dirname, '../../resources');
        this._bPackageConfig = false;
    }
    AbstractCommand.prototype.validateWchToolsOptions = function (aOpt) {
        return !!(aOpt && aOpt.baseUrl && aOpt.username && aOpt.password);
    };
    AbstractCommand.prototype.getWchTools = function () {
        if (!this._wchTools) {
            this._wchTools = this.rxMkdirp(this.getDataDir()).pipe(operators_1.mergeMap(wchtools_1.readWchTools), operators_1.shareReplay());
        }
        return this._wchTools;
    };
    AbstractCommand.prototype.getWchToolsOptions = function () {
        var _this = this;
        if (!this._wchToolsOptions) {
            this._wchToolsOptions = wchtools_1.wchToolsGetOptions(this.getOptions(), this.getDataDir()).pipe(operators_1.tap(function (opt) {
                if (!_this.validateWchToolsOptions(opt)) {
                    throw new Error('Insufficient Credentials');
                }
            }), operators_1.shareReplay());
        }
        return this._wchToolsOptions;
    };
    AbstractCommand.prototype.getAngularCliConfig = function () {
        if (!this._angularJson) {
            this._angularJson = rx_file_1.rxReadTextFile(path_1.join(this.getPackageDir(), 'angular.json')).pipe(operators_1.map(function (file) { return JSON.parse(file); }));
        }
        // ok
        return this._angularJson;
    };
    AbstractCommand.prototype.getContextRoot = function (aAppName) {
        if (!this._contextRoot) {
            // do we have a context root
            var configRoot = config_1.configGetContextRoot(this.getPackageDir(), this.getPackageConfig());
            // if not specified, use the application name
            if ((configRoot === null) || (configRoot === undefined)) {
                // fallback to using the application name
                this._contextRoot = "/" + encodeURIComponent(aAppName || this.getAppName());
            }
            else {
                // use the specified root
                this._contextRoot = "/" + config_1.configTrimSlashes(configRoot);
            }
            // log this
            var logger = this.getLogger();
            logger.next("AppContextRoot [" + this._contextRoot + "].");
        }
        // ok
        return this._contextRoot;
    };
    AbstractCommand.prototype.getAppAssetDir = function (aAppName) {
        if (!this._appAssetDir) {
            this._appAssetDir = path.resolve(path.normalize(path.join(this.getDataDir(), 'assets', config_1.configTrimSlashes(aAppName || this.getAppName()))));
            // log this
            var logger = this.getLogger();
            logger.next("AppAssetDirectory [" + this._appAssetDir + "].");
        }
        // ok
        return this._appAssetDir;
    };
    AbstractCommand.prototype.getApiBase = function () {
        var _this = this;
        if (!this._apiBase) {
            this._apiBase = wchtools_1.wchToolsGetApiBaseUrl(this.getOptions(), this.getDataDir()).pipe(operators_1.tap(function (url) { return _this.getLogger().next("ApiBaseUrl [" + url + "]."); }), operators_1.shareReplay());
        }
        // ok
        return this._apiBase;
    };
    AbstractCommand.prototype.getDeliveryBase = function () {
        var _this = this;
        if (!this._deliveryBase) {
            this._deliveryBase = this.getApiBase().pipe(operators_1.map(wchtools_1.wchDeliveryUrlFromApiUrl), operators_1.tap(function (url) { return _this.getLogger().next("DeliveryBaseUrl [" + url + "]."); }), operators_1.shareReplay());
        }
        // ok
        return this._deliveryBase;
    };
    AbstractCommand.prototype.getPackageConfig = function () {
        if (!this._bPackageConfig) {
            this._packageConfig = config_1.configReadPackageConfig(this.getPackageDir());
            this._bPackageConfig = true;
        }
        // ok
        return this._packageConfig;
    };
    AbstractCommand.prototype.getAppName = function () {
        if (!this._appName) {
            // check for the option
            var opts = this.getOptions();
            this._appName = config_1.configGetAppName(this.getPackageDir(), this.getPackageConfig()) || 'defaultApp';
            // log this
            var logger = this.getLogger();
            logger.next("AppName [" + this._appName + "].");
        }
        // ok
        return this._appName;
    };
    AbstractCommand.prototype.getAppDir = function () {
        if (!this._appDir) {
            // check for the option
            var opts = this.getOptions();
            this._appDir = config_1.configGetAppDir(this.getPackageDir(), this.getPackageConfig()) || 'defaultApp';
            // log this
            var logger = this.getLogger();
            logger.next("AppDirectory [" + this._appDir + "].");
        }
        // ok
        return this._appDir;
    };
    AbstractCommand.prototype.getConfigValue = function (aKey) {
        var config = this.getPackageConfig();
        return config ? config[aKey] : undefined;
    };
    AbstractCommand.prototype.getDataDir = function () {
        if (!this._dataDir) {
            // check for the option
            var opts = this.getOptions();
            this._dataDir = path.resolve(opts.data || config_1.configGetDataDir(this.getPackageDir(), this.getPackageConfig()) || '.');
            // log this
            var logger = this.getLogger();
            logger.next("DataDirectory [" + this._dataDir + "].");
        }
        // ok
        return this._dataDir;
    };
    AbstractCommand.prototype.getPackageDir = function () {
        if (!this._packageDir) {
            // check if we have the override
            var opts = this.getOptions();
            var pkgDir = opts.pkgDir || '.';
            this._packageDir = path.resolve(path.normalize(pkgDir));
            // log this
            var logger = this.getLogger();
            logger.next("PackageDirectory [" + this._packageDir + "].");
        }
        // ok
        return this._packageDir;
    };
    AbstractCommand.prototype.getCommandName = function () {
        return this.constructor.name;
    };
    AbstractCommand.prototype.printHelp = function () {
    };
    AbstractCommand.prototype.getLogger = function () {
        if (!this._logger) {
            // logger
            var logger = new rxjs_1.Subject();
            var name_1 = this.getCommandName();
            // add some logging
            if (this.abstractOptions.verbose) {
                logger.subscribe(function (data) { return console.info("\u001B[32m " + name_1, data, '\x1b[0m'); });
            }
            // attach the logger
            this._logger = logger;
        }
        // ok
        return this._logger;
    };
    AbstractCommand.prototype.getOptions = function () {
        return this.abstractOptions;
    };
    AbstractCommand.prototype.getFileName = function (aIdentifier) {
        return transliteration_1.kebabCase(transliteration_1.camelCase(aIdentifier));
    };
    AbstractCommand.prototype.rxMkdirp = function (aPath) {
        return rx_walk_1.rxMkdirp(aPath).pipe(operators_1.shareReplay());
    };
    return AbstractCommand;
}());
exports.AbstractCommand = AbstractCommand;
