"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
exports.__esModule = true;
/* Copyright IBM Corp. 2017 */
var path_1 = require("path");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var verror_1 = require("verror");
var constants_1 = require("../constants");
var asset_dependencies_1 = require("../utils/asset.dependencies");
var rx_copy_1 = require("../utils/rx.copy");
var rx_delete_1 = require("../utils/rx.delete");
var rx_walk_1 = require("./../utils/rx.walk");
var wchtools_1 = require("./../utils/wchtools");
var abstract_command_1 = require("./abstract.command");
var REMOVE_KEY_VALUE_ARGS = [constants_1.OPTION_SOURCE, constants_1.OPTION_DATA, constants_1.OPTION_USER, constants_1.OPTION_URL, constants_1.OPTION_PASSWORD, constants_1.OPTION_PKG_DIR];
var REMOVE_KEY_ARGS = [constants_1.OPTION_VERBOSE, constants_1.OPTION_AGGREGATED];
/**
 * Remove arguments that we interpreted
 *
 * @param aArgs   args
 */
function _fixArguments(aArgs) {
    // target
    var result = [];
    var len = aArgs.length;
    var idx = 0;
    while (idx < len) {
        // check the current argument
        var arg = aArgs[idx++];
        // check if this is a key
        if (arg.startsWith('--')) {
            // argument key
            var key = arg.substr(2);
            if (REMOVE_KEY_VALUE_ARGS.indexOf(key) >= 0) {
                // remove key and value
                idx++;
            }
            else if (REMOVE_KEY_ARGS.indexOf(key) >= 0) {
                // noop
            }
            else {
                // add the argument
                result.push(arg);
            }
        }
        else {
            // add the argument
            result.push(arg);
        }
    }
    // o
    return result;
}
exports.wchtoolsFixArguments = _fixArguments;
var AbstractWchToolsCommand = /** @class */ (function (_super) {
    __extends(AbstractWchToolsCommand, _super);
    function AbstractWchToolsCommand(aWchToolsOptions) {
        var _this = _super.call(this, aWchToolsOptions) || this;
        _this.aWchToolsOptions = aWchToolsOptions;
        return _this;
    }
    AbstractWchToolsCommand.prototype.getWchToolsOptions = function () {
        // lazy load the options
        if (!this._wchToolsOpts) {
            this._wchToolsOpts = wchtools_1.wchToolsGetOptions(this.getOptions(), this.getDataDir()).pipe(operators_1.shareReplay(1));
        }
        // returns the options
        return this._wchToolsOpts;
    };
    AbstractWchToolsCommand.prototype.executeWchTools = function (aArgs) {
        var _this = this;
        // check if we are in aggregated mode
        var bAggregated = !!this.aWchToolsOptions.aggregated;
        if (bAggregated) {
            // find the package dir
            var pkgDir = this.getPackageDir();
            var pkgFile_1 = path_1.join(pkgDir, 'package.json');
            // copy the dependencies
            var rxDstBase = rx_walk_1.rxTempDir().pipe(operators_1.shareReplay(1));
            var logger_1 = this.getLogger();
            // clean up after execution
            var rxDelete_1 = rxDstBase.pipe(operators_1.mergeMap(function (dstBase) { return rx_delete_1.rxEmptyDir(dstBase).pipe(operators_1.catchError(function (error) { return rxjs_1.throwError(new verror_1.VError(error, 'Unable to clean the temp directory [%s].', dstBase)); }), operators_1.tap(rxjs_1.noop, rxjs_1.noop, function () { return logger_1.next("Deleted temp directory [" + dstBase + "]."); })); }));
            // copy the dependencies
            var rxCopy = rxDstBase.pipe(operators_1.mergeMap(function (dstBase) { return asset_dependencies_1.rxCopyDependencies(pkgFile_1, dstBase).pipe(operators_1.catchError(function (error) { return rxjs_1.throwError(new verror_1.VError(error, 'Unable to copy the aggregated data from [%s] to [%s].', pkgFile_1, dstBase)); }), operators_1.tap(rxjs_1.noop, rxjs_1.noop, function () { return logger_1.next("Copied the aggregated data from [" + pkgFile_1 + "] to [" + dstBase + "]."); })); }));
            // copy our own data dir
            var dataDir_1 = this.getDataDir();
            var rxCopyData = rxDstBase.pipe(operators_1.mergeMap(function (dstBase) { return rx_copy_1.rxCopyDir(dataDir_1, dstBase, true).pipe(operators_1.catchError(function (error) { return rxjs_1.throwError(new verror_1.VError(error, 'Unable to copy the application data from [%s] to [%s].', dataDir_1, dstBase)); }), operators_1.tap(rxjs_1.noop, rxjs_1.noop, function () { return logger_1.next("Copied the application data from [" + dataDir_1 + "] to [" + dstBase + "]."); })); }));
            var wchtoolsArgs_1 = _fixArguments(aArgs);
            var rxWchTools = rxDstBase.pipe(operators_1.mergeMap(function (dstBase) { return _this.getWchToolsOptions().pipe(operators_1.mergeMap(function (opts) { return wchtools_1.wchExecuteWchTools(dstBase, wchtoolsArgs_1, _this.aWchToolsOptions, opts, logger_1); }), operators_1.catchError(function (error) { return rxjs_1.throwError(new verror_1.VError(error, 'Unable to execute wchtools with arguments [%s].', JSON.stringify(wchtools_1.wchHidePassword(wchtoolsArgs_1)))); }), operators_1.tap(rxjs_1.noop, rxjs_1.noop, function () { return logger_1.next("Executed wchtools."); })); }));
            // execute
            return rxjs_1.concat(rxDelete_1, rxCopy, rxCopyData, rxWchTools, rxDelete_1).pipe(operators_1.distinct(), operators_1.catchError(function (error) { return rxDelete_1.pipe(operators_1.mergeMapTo(rxjs_1.throwError(new verror_1.VError(error, 'Unable to push the application.')))); }), operators_1.tap(rxjs_1.noop, rxjs_1.noop, function () { return logger_1.next("Command complete."); }));
        }
        // default
        return this.getWchToolsOptions().pipe(operators_1.mergeMap(function (opts) { return wchtools_1.wchExecuteWchTools(_this.getDataDir(), _fixArguments(aArgs), _this.aWchToolsOptions, opts, _this.getLogger()); }));
    };
    return AbstractWchToolsCommand;
}(abstract_command_1.AbstractCommand));
exports.AbstractWchToolsCommand = AbstractWchToolsCommand;
