"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
exports.__esModule = true;
/* Copyright IBM Corp. 2017 */
var lodash_1 = require("lodash");
var path_1 = require("path");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var url_1 = require("url");
var file_utils_1 = require("../utils/file.utils");
var rx_utils_1 = require("../utils/rx.utils");
var constants_1 = require("./../constants");
var options_1 = require("./../options");
var config_1 = require("./../utils/config");
var rx_walk_1 = require("./../utils/rx.walk");
var url_utils_1 = require("./../utils/url.utils");
var wchtools_1 = require("./../utils/wchtools");
var abstract_command_1 = require("./abstract.command");
var help_1 = require("./help");
function _createUrl(aContextRoot, aApiBaseUrl, aRelative) {
    // the delivery URL
    var absWchDeliveryUrl = url_utils_1.ensureTrailingSlash(wchtools_1.wchDeliveryUrlFromApiUrl(aApiBaseUrl));
    // path of the index file
    var absRootUrl = url_utils_1.ensureTrailingSlash("" + absWchDeliveryUrl + config_1.configTrimSlashes(aContextRoot));
    // parse the URL
    if (!aRelative) {
        return rxjs_1.of(absRootUrl);
    }
    // make relative
    var parsed = url_1.parse(absRootUrl);
    return rxjs_1.of(parsed.path || '/');
}
function _fixIndex(aSource, aUrl, aLogger) {
    // log the settings
    aLogger.next("BaseURL [" + aUrl + "].");
    // insert the layouts import
    var src = aSource;
    // add the module to the end of the declarations
    src = src.replace(/(<base\s+href=\")([^\"]*)(\">)/g, '$1' + lodash_1.escape(aUrl) + '$3');
    return src;
}
function _rewriteIndex(aSrcDir, aContextRoot, aApiBaseUrl, aRelative, aLogger) {
    // compute the proper base URL
    var baseUrl = _createUrl(aContextRoot, aApiBaseUrl, aRelative);
    // name of the index file
    var name = path_1.join(aSrcDir, 'index.html');
    return rx_walk_1.readFile(name).pipe(operators_1.mergeMap(function (src) { return baseUrl.pipe(operators_1.map(function (url) { return _fixIndex(src, url, aLogger); })); }), operators_1.mergeMap(function (data) { return rx_walk_1.writeFile(name, data); }));
}
/**
 * Generates layout and layout mappings
 *
 * add base --data DATA_FOLDER --src SRC_FOLDER [--relative]
 */
var AddBaseCommand = /** @class */ (function (_super) {
    __extends(AddBaseCommand, _super);
    function AddBaseCommand(options) {
        return _super.call(this, options) || this;
    }
    AddBaseCommand.prototype.printHelp = function () {
        // log the command line
        console.log(AddBaseCommand.COMMAND, AddBaseCommand.TYPE, "[--" + constants_1.OPTION_SOURCE + " <SOURCE>]", "[--" + constants_1.OPTION_DATA + " <DATA>]", "[--" + constants_1.OPTION_URL + " <URL>]", "[--" + constants_1.OPTION_NAME + " <NAME>]", "[--" + constants_1.OPTION_ROOT + " <ROOT>]", "[--" + constants_1.OPTION_RELATIVE + "]", "[--" + constants_1.OPTION_VERBOSE + "]");
        // help on the options
        console.log("--" + constants_1.OPTION_SOURCE + " <SOURCE>", ':', help_1.HELP_INDEX_SOURCE);
        console.log("--" + constants_1.OPTION_DATA + " <DATA>", ':', help_1.HELP_DATA_DIRECTORY);
        console.log("--" + constants_1.OPTION_URL + " <URL>", ':', help_1.HELP_API_URL);
        console.log("--" + constants_1.OPTION_NAME + " <NAME>", ':', help_1.HELP_APPLICATION_NAME);
        console.log("--" + constants_1.OPTION_ROOT, ':', help_1.HELP_CONTEXT_ROOT);
        console.log("--" + constants_1.OPTION_VERBOSE, ':', help_1.HELP_VERBOSE);
        console.log("--" + constants_1.OPTION_RELATIVE, ':', help_1.HELP_RELATIVE);
    };
    AddBaseCommand.prototype.process = function () {
        // options
        var opts = this.getOptions();
        var dataDirOpt = this.getDataDir();
        var appName = opts.name || this.getAppName();
        var contextRoot = "/" + config_1.configTrimSlashes(opts.root || this.getContextRoot(appName));
        // the directories
        var srcDirOpt = options_1.optNormalizeDir(file_utils_1.firstOf(opts.src) || config_1.configIndexDirFromContextRoot(contextRoot, dataDirOpt));
        var srcDir = this.rxMkdirp(srcDirOpt);
        // base URL
        var apiBaseUrl = this.getApiBase();
        // check if we should use a relative URL
        var bRelative = !!opts.relative;
        // the logger
        var logger = this.getLogger();
        // log the settings
        logger.next("Source dir [" + srcDirOpt + "].");
        // wait for the directories
        return rx_utils_1.rxForkJoin([srcDir, apiBaseUrl]).pipe(operators_1.mergeMap(function (_a) {
            var aSrcDir = _a[0], aBaseUrl = _a[1];
            return _rewriteIndex(aSrcDir, contextRoot, aBaseUrl, bRelative, logger);
        }));
    };
    AddBaseCommand.TYPE = 'base';
    AddBaseCommand.COMMAND = constants_1.CMD_ADD;
    return AddBaseCommand;
}(abstract_command_1.AbstractCommand));
exports.AddBaseCommand = AddBaseCommand;
