"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
exports.__esModule = true;
/* Copyright IBM Corp. 2017 */
var path_1 = require("path");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var file_utils_1 = require("../utils/file.utils");
var constants_1 = require("./../constants");
var options_1 = require("./../options");
var config_1 = require("./../utils/config");
var rx_hbs_1 = require("./../utils/rx.hbs");
var rx_walk_1 = require("./../utils/rx.walk");
var abstract_command_1 = require("./abstract.command");
var help_1 = require("./help");
function _getPolyfillTemplate() {
    return rx_hbs_1.hbsCompile(path_1.join(__dirname, '..', '..', '..', 'assets', 'esi/esi.hbs'));
}
var HEAD_REGEXP = /<\/head>/g;
function _insertMarkup(aPolyfill, aMarkup) {
    // replace the head regular expression
    return aMarkup.replace(HEAD_REGEXP, aPolyfill + "\n</head>");
}
function _getPolyfillScript() {
    return rx_walk_1.readFile(path_1.join(__dirname, '..', '..', '..', 'assets', 'esi/esi.min.js'));
}
function _rewriteIndex(aSrcDir) {
    // load the polyfill script as context
    var rxPolyfillContext = _getPolyfillScript().pipe(operators_1.map(function (script) { return ({
        SCRIPT: script
    }); }));
    // load the markup template
    var rxPolyfillTemplate = _getPolyfillTemplate();
    // excute
    var rxMarkup = rxjs_1.combineLatest(rxPolyfillContext, rxPolyfillTemplate).pipe(operators_1.map(function (_a) {
        var ctx = _a[0], tmp = _a[1];
        return tmp(ctx);
    }));
    // read the index file
    var fileName = path_1.join(aSrcDir, 'index.html');
    return rxjs_1.combineLatest(rx_walk_1.readFile(fileName), rxMarkup).pipe(operators_1.map(function (_a) {
        var index = _a[0], markup = _a[1];
        return _insertMarkup(markup, index);
    }), operators_1.mergeMap(function (newMarkup) { return rx_walk_1.writeFile(fileName, newMarkup); }));
}
var AddEsiPolyfillCommand = /** @class */ (function (_super) {
    __extends(AddEsiPolyfillCommand, _super);
    function AddEsiPolyfillCommand(options) {
        var _this = _super.call(this, options) || this;
        _this.options = options;
        return _this;
    }
    AddEsiPolyfillCommand.prototype.printHelp = function () {
        // log the command line
        console.log(AddEsiPolyfillCommand.COMMAND, AddEsiPolyfillCommand.TYPE, "[--" + constants_1.OPTION_SOURCE + " <SOURCE>]", "[--" + constants_1.OPTION_DATA + " <DATA>]", "[--" + constants_1.OPTION_NAME + " <NAME>]", "[--" + constants_1.OPTION_ROOT + " <ROOT>]", "[--" + constants_1.OPTION_VERBOSE + "]");
        // help on the options
        console.log("--" + constants_1.OPTION_SOURCE + " <SOURCE>", ':', help_1.HELP_INDEX_SOURCE);
        console.log("--" + constants_1.OPTION_DATA + " <DATA>", ':', help_1.HELP_DATA_DIRECTORY);
        console.log("--" + constants_1.OPTION_NAME + " <NAME>", ':', help_1.HELP_APPLICATION_NAME);
        console.log("--" + constants_1.OPTION_ROOT, ':', help_1.HELP_CONTEXT_ROOT);
        console.log("--" + constants_1.OPTION_VERBOSE, ':', help_1.HELP_VERBOSE);
    };
    AddEsiPolyfillCommand.prototype.process = function () {
        // logger
        var logger = this.getLogger();
        // data directory
        var opts = this.options;
        // the directories
        var dataDirOpt = this.getDataDir();
        var appName = opts.name || this.getAppName();
        var contextRoot = "/" + config_1.configTrimSlashes(opts.root || this.getContextRoot(appName));
        // the directories
        var srcDirOpt = options_1.optNormalizeDir(file_utils_1.firstOf(opts.src) || config_1.configIndexDirFromContextRoot(contextRoot, dataDirOpt));
        var rxSrcDir = this.rxMkdirp(srcDirOpt);
        // rewrite
        return rxSrcDir.pipe(operators_1.mergeMap(_rewriteIndex));
    };
    AddEsiPolyfillCommand.TYPE = 'esi-polyfill';
    AddEsiPolyfillCommand.COMMAND = constants_1.CMD_ADD;
    return AddEsiPolyfillCommand;
}(abstract_command_1.AbstractCommand));
exports.AddEsiPolyfillCommand = AddEsiPolyfillCommand;
