"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
exports.__esModule = true;
var lodash_1 = require("lodash");
var path_1 = require("path");
var RelateUrl = require("relateurl");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var url_1 = require("url");
var file_utils_1 = require("../utils/file.utils");
var constants_1 = require("./../constants");
var options_1 = require("./../options");
var config_1 = require("./../utils/config");
var rx_walk_1 = require("./../utils/rx.walk");
var url_utils_1 = require("./../utils/url.utils");
var abstract_command_1 = require("./abstract.command");
var create_webappmanifest_command_1 = require("./create.webappmanifest.command");
var help_1 = require("./help");
/* Copyright IBM Corp. 2017 */
var HEAD_REGEXP = /<\/head>/g;
var FAVICON_REGEXP = /<link\s+[^>]*rel\s*=\s*\"icon\"[^>]*>[\s|\r|\n]*/g;
var MANIFEST_REGEXP = /<link\s+[^>]*rel\s*=\s*\"manifest\"[^>]*>[\s|\r|\n]*/g;
function _createUrl(aBaseUrl, aDeliveryBase, aRelUrl, bRelative) {
    // parse
    var url = "" + aDeliveryBase + aRelUrl;
    var parsed = url_1.parse(url);
    return (bRelative ? aBaseUrl.relate(url) : "//" + parsed.host + parsed.path) || url;
}
/**
 * Creates the links to the manifest file and the favicons
 *
 * @param aDeliveryBase base URL
 * @param aManifestType manifest type
 * @param bRelative check if the links should be relative or absolute
 * @param aLogger   the logger
 */
function _createLinks(aDeliveryBase, aContextRoot, aManifestType, bRelative, bEsi, aLogger) {
    // log this
    aLogger.next("Delivery base [" + aDeliveryBase + "].");
    // the base URL
    var absRootUrl = url_utils_1.ensureTrailingSlash("" + aDeliveryBase + config_1.configTrimSlashes(aContextRoot));
    // base URL
    var baseUrl = new RelateUrl(absRootUrl);
    // URL to the manifest
    var manifestUrl = _createUrl(baseUrl, aDeliveryBase, "templates/manifest/" + aManifestType + "_manifest.json", bRelative);
    // URL to the favicons
    var faviconsUrl = _createUrl(baseUrl, aDeliveryBase, "templates/favicons/" + aManifestType + "_favicons.html", bRelative);
    // produce the markup
    var links = [];
    links.push("<link rel=\"manifest\" href=\"" + lodash_1.escape(manifestUrl) + "\">");
    // check if we would create ESI or not
    if (bEsi) {
        links.push("<esi:include src=\"" + lodash_1.escape(faviconsUrl) + "\"></esi:include>");
    }
    else {
        links.push("<link rel=\"esi:include\" href=\"" + lodash_1.escape(faviconsUrl) + "\">");
    }
    // ok
    return links.join('\n');
}
function _insertMarkup(aFavicons, aMarkup) {
    // delete an existing icon
    var html = aMarkup.replace(FAVICON_REGEXP, '').replace(MANIFEST_REGEXP, '');
    // replace the head regular expression
    html = html.replace(HEAD_REGEXP, aFavicons + "\n</head>");
    // ok
    return html;
}
function _rewriteIndex(aMarkup, aSrcDir) {
    // read the index file
    var fileName = path_1.join(aSrcDir, 'index.html');
    return rx_walk_1.readFile(fileName).pipe(operators_1.map(function (content) { return _insertMarkup(aMarkup, content); }), operators_1.mergeMap(function (newMarkup) { return rx_walk_1.writeFile(fileName, newMarkup); }));
}
function _addWebAppManifest(aDeliveryBase, aContextRoot, aManifestType, bRelative, bEsi, aSrcDir, aLogger) {
    // construct the markup
    var markup = _createLinks(aDeliveryBase, aContextRoot, aManifestType, bRelative, bEsi, aLogger);
    // rewrite
    return _rewriteIndex(markup, aSrcDir);
}
var AddWebAppManifestCommand = /** @class */ (function (_super) {
    __extends(AddWebAppManifestCommand, _super);
    function AddWebAppManifestCommand(options) {
        return _super.call(this, options) || this;
    }
    AddWebAppManifestCommand.prototype.printHelp = function () {
        // log the command line
        console.log(AddWebAppManifestCommand.COMMAND, AddWebAppManifestCommand.TYPE, "[--" + constants_1.OPTION_SOURCE + " <SOURCE>]", "[--" + constants_1.OPTION_DATA + " <DATA>]", "[--" + constants_1.OPTION_URL + " <URL>]", "[--" + constants_1.OPTION_NAME + " <NAME>]", "[--" + constants_1.OPTION_ROOT + " <ROOT>]", "[--" + constants_1.OPTION_RELATIVE + "]", "[--" + constants_1.OPTION_ESI + "]", "[--" + constants_1.OPTION_VERBOSE + "]");
        // help on the options
        console.log("--" + constants_1.OPTION_SOURCE + " <SOURCE>", ':', help_1.HELP_INDEX_SOURCE);
        console.log("--" + constants_1.OPTION_DATA + " <DATA>", ':', help_1.HELP_DATA_DIRECTORY);
        console.log("--" + constants_1.OPTION_URL + " <URL>", ':', help_1.HELP_API_URL);
        console.log("--" + constants_1.OPTION_NAME + " <NAME>", ':', help_1.HELP_APPLICATION_NAME);
        console.log("--" + constants_1.OPTION_ROOT, ':', help_1.HELP_CONTEXT_ROOT);
        console.log("--" + constants_1.OPTION_VERBOSE, ':', help_1.HELP_VERBOSE);
        console.log("--" + constants_1.OPTION_RELATIVE, ':', help_1.HELP_RELATIVE);
        console.log("--" + constants_1.OPTION_ESI, ':', help_1.HELP_ESI);
    };
    AddWebAppManifestCommand.prototype.process = function () {
        // logger
        var logger = this.getLogger();
        // data directory
        var opts = this.getOptions();
        // the directories
        var dataDirOpt = this.getDataDir();
        var appName = opts.name || this.getAppName();
        var contextRoot = "/" + config_1.configTrimSlashes(opts.root || this.getContextRoot(appName));
        // the directories
        var srcDirOpt = options_1.optNormalizeDir(file_utils_1.firstOf(opts.src) || config_1.configIndexDirFromContextRoot(contextRoot, dataDirOpt));
        var rxSrcDir = this.rxMkdirp(srcDirOpt);
        // the url
        var rxDeliveryBase = this.getDeliveryBase();
        // type id
        var manifestId = opts.id || this.getConfigValue(create_webappmanifest_command_1.KEY_MANIFEST_ID) || '';
        // relative or absolute
        var bRelative = !!opts.relative;
        // ESI
        var bEsi = !!opts.esi;
        // proceed
        logger.next("Source dir [" + srcDirOpt + "], manifestid [" + manifestId + "], relative [" + bRelative + "], esi [" + bEsi + "].");
        // proceed
        return rxjs_1.combineLatest(rxSrcDir, rxDeliveryBase).pipe(operators_1.mergeMap(function (_a) {
            var aSrcDir = _a[0], aDeliveryBase = _a[1];
            return _addWebAppManifest(aDeliveryBase, contextRoot, manifestId, bRelative, bEsi, aSrcDir, logger);
        }));
    };
    AddWebAppManifestCommand.TYPE = 'webappmanifest';
    AddWebAppManifestCommand.COMMAND = constants_1.CMD_ADD;
    return AddWebAppManifestCommand;
}(abstract_command_1.AbstractCommand));
exports.AddWebAppManifestCommand = AddWebAppManifestCommand;
