"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
exports.__esModule = true;
/* Copyright IBM Corp. 2017 */
var Assert = require("assert");
var lodash_1 = require("lodash");
var RelateUrl = require("relateurl");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var wchtools_1 = require("../utils/wchtools");
var constants_1 = require("./../constants");
var config_1 = require("./../utils/config");
var abstract_wchtools_command_1 = require("./abstract.wchtools.command");
var help_1 = require("./help");
function assertHasContext(aValue) {
    Assert.ok(lodash_1.isString(aValue) && (aValue.length > 0), 'Application has to use a context root.');
    return aValue;
}
exports.assertHasContext = assertHasContext;
var ApplicationPurgeCommand = /** @class */ (function (_super) {
    __extends(ApplicationPurgeCommand, _super);
    function ApplicationPurgeCommand(aOptions) {
        return _super.call(this, aOptions) || this;
    }
    ApplicationPurgeCommand.prototype.printHelp = function () {
        // log the command line
        console.log(ApplicationPurgeCommand.COMMAND, ApplicationPurgeCommand.TYPE, "--" + constants_1.OPTION_DATA + " <DATA>", "[--" + constants_1.OPTION_NAME + " <NAME>]", "[--" + constants_1.OPTION_URL + " <URL>]", "[--" + constants_1.OPTION_USER + " <USER>]", "[--" + constants_1.OPTION_PASSWORD + " <PASSWORD>]", "[--" + constants_1.OPTION_VERBOSE + "]");
        // log the parameters
        console.log("--" + constants_1.OPTION_NAME + " <NAME>", ':', help_1.HELP_APPLICATION_NAME);
        console.log("--" + constants_1.OPTION_DATA + " <DATA>", ':', help_1.HELP_DATA_DIRECTORY);
        console.log("--" + constants_1.OPTION_URL + " <URL>", ':', help_1.HELP_API_URL);
        console.log("--" + constants_1.OPTION_USER + " <USER>", ':', help_1.HELP_USERNAME);
        console.log("--" + constants_1.OPTION_PASSWORD + " <PASSWORD>", ':', help_1.HELP_PASSWORD);
        console.log("--" + constants_1.OPTION_VERBOSE, ':', help_1.HELP_VERBOSE);
    };
    ApplicationPurgeCommand.prototype.process = function () {
        var _this = this;
        // the options
        var opts = this.getOptions();
        // options
        var appName = opts.name || this.getAppName();
        var contextRoot = "/" + config_1.configTrimSlashes(opts.root || this.getContextRoot(appName));
        // api base
        var rxApiBaseUrl = this.getApiBase().pipe(operators_1.shareReplay());
        // the delivery URL
        var rxAbsWchDeliveryUrl = rxApiBaseUrl.pipe(operators_1.map(function (aApiBaseUrl) { return wchtools_1.ensureTrailingSlash(wchtools_1.wchDeliveryUrlFromApiUrl(aApiBaseUrl)); }), operators_1.shareReplay());
        // path of the index file
        var rxAbsRootUrl = rxAbsWchDeliveryUrl.pipe(operators_1.map(function (absWchDeliveryUrl) { return wchtools_1.ensureTrailingSlash("" + absWchDeliveryUrl + config_1.configTrimSlashes(contextRoot)); }), operators_1.shareReplay());
        // get the base URL
        var rxAbsAppUrl = rxApiBaseUrl.pipe(operators_1.map(function (aApiBaseUrl) { return wchtools_1.ensureTrailingSlash(config_1.configGetBaseUrl(appName, aApiBaseUrl)); }), operators_1.shareReplay());
        // relative delivery
        var rxDeliveryUrl = rxjs_1.combineLatest(rxAbsRootUrl, rxAbsAppUrl).pipe(operators_1.map(function (_a) {
            var absRootUrl = _a[0], absAppUrl = _a[1];
            return new RelateUrl(absRootUrl).relate(absAppUrl);
        }), operators_1.shareReplay());
        // construct the url to delete
        var rxPathToDelete = rxDeliveryUrl.pipe(operators_1.map(config_1.configTrimSlashes), operators_1.map(assertHasContext), operators_1.filter(Boolean), operators_1.map(function (url) { return '/' + url; }), operators_1.shareReplay());
        // construct the command line arguments
        var rxArgs = rxPathToDelete.pipe(operators_1.map(function (path) { return ['delete', '-w', '--recursive', '--path', path, '--quiet']; }));
        // execute
        return rxArgs.pipe(operators_1.switchMap(function (args) { return _this.executeWchTools(args); }), operators_1.catchError(function (err) { return rxjs_1.empty(); }));
    };
    ApplicationPurgeCommand.TYPE = 'purge';
    ApplicationPurgeCommand.COMMAND = constants_1.CMD_APP;
    return ApplicationPurgeCommand;
}(abstract_wchtools_command_1.AbstractWchToolsCommand));
exports.ApplicationPurgeCommand = ApplicationPurgeCommand;
