/* Copyright IBM Corp. 2017 */
import { Observable } from 'rxjs';

import {
    CMD_APP,
    OPTION_AGGREGATED,
    OPTION_DATA,
    OPTION_OVERRIDE,
    OPTION_PASSWORD,
    OPTION_URL,
    OPTION_USER,
    OPTION_VERBOSE,
} from './../constants';
import { Options } from './../options';
import { AbstractWchToolsCommand } from './abstract.wchtools.command';
import {
    HELP_AGGREGATED,
    HELP_API_URL,
    HELP_DATA_DIRECTORY,
    HELP_PASSWORD,
    HELP_PROJECT_OVERRIDE,
    HELP_USERNAME,
    HELP_VERBOSE,
} from './help';

/**
 * application push --data DATA_FOLDER
 */
export class ApplicationPushCommand extends AbstractWchToolsCommand {

    static readonly TYPE = 'push';

    static readonly COMMAND = CMD_APP;

    constructor(aOptions: Options) {
        super(aOptions);
    }

    printHelp(): void {
        // log the command line
        console.log(ApplicationPushCommand.COMMAND, ApplicationPushCommand.TYPE,
            `--${OPTION_DATA} <DATA>`,
            `[--${OPTION_URL} <URL>]`,
            `[--${OPTION_USER} <USER>]`,
            `[--${OPTION_PASSWORD} <PASSWORD>]`,
            `[--${OPTION_AGGREGATED}]`,
            `[--${OPTION_OVERRIDE}]`,
            `[--${OPTION_VERBOSE}]`);
        // log the parameters
        console.log(`--${OPTION_DATA}`, ':', HELP_DATA_DIRECTORY);
        console.log(`--${OPTION_URL}`, ':', HELP_API_URL);
        console.log(`--${OPTION_USER}`, ':', HELP_USERNAME);
        console.log(`--${OPTION_PASSWORD}`, ':', HELP_PASSWORD);
        console.log(`--${OPTION_OVERRIDE}`, ':', HELP_PROJECT_OVERRIDE);
        console.log(`--${OPTION_AGGREGATED}`, ':', HELP_AGGREGATED);
        console.log(`--${OPTION_VERBOSE}`, ':', HELP_VERBOSE);
    }

    process(): Observable<string> {
        // the options
        const opts = this.getOptions();
        // override?
        const bOverride = !!opts.override;
        // add the override commands if required
        const args = ['push', '-tlmwaicrCp'];
        if (bOverride) {
            args.push('-fI');
        }
        // dispatch
        return this.executeWchTools(args);
    }
}
