/* Copyright IBM Corp. 2017 */
import { getType } from 'mime';
import { Observable } from 'rxjs';
import { filter, map, mergeMap } from 'rxjs/operators';

import { canonicalizeJSON } from '../utils/json';
import { CMD_CANONICALIZE } from './../constants';
import { Options } from './../options';
import { JsonFile, readJSON, writeJSON } from './../utils/rx.json';
import { walkFiles } from './../utils/rx.walk';
import { AbstractCommand } from './abstract.command';
import { Command } from './command';

function _cleanupJSON(aFile: JsonFile): JsonFile {
    aFile.data = canonicalizeJSON(aFile.data);
    return aFile;
}

export class CanonicalizeAssetsCommand extends AbstractCommand implements Command {

    static readonly TYPE = 'assets';
    static readonly COMMAND = CMD_CANONICALIZE;

    constructor(options: Options) {
        super(options);
    }

    process(): Observable<string> {
        // data dir
        const rxDataDir = this.rxMkdirp(this.getDataDir());

        // process
        return rxDataDir.pipe(
            mergeMap(walkFiles),
            filter(f => getType(f.absPath) === 'application/json'),
            mergeMap(readJSON),
            map(_cleanupJSON),
            mergeMap(writeJSON),
            map(file => file.absPath)
        );
    }
}

