"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
exports.__esModule = true;
/* Copyright IBM Corp. 2017 */
var path_1 = require("path");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var asset_dependencies_1 = require("../utils/asset.dependencies");
var rx_file_1 = require("../utils/rx.file");
var rx_walk_1 = require("../utils/rx.walk");
var constants_1 = require("./../constants");
var options_1 = require("./../options");
var rx_copy_1 = require("./../utils/rx.copy");
var rx_delete_1 = require("./../utils/rx.delete");
var wchtools_1 = require("./../utils/wchtools");
var abstract_command_1 = require("./abstract.command");
var help_1 = require("./help");
function _copyAggregated(aPackageFile, aDstDir, aLogger) {
    var pkgFile = aPackageFile;
    // copy the dependencies
    var dstBase = aDstDir;
    aLogger.next("Aggregating to [" + dstBase + "].");
    // clean up after execution
    var rxDelete = rx_delete_1.rxDeleteDir(dstBase);
    // copy the dependencies
    var rxCopy = asset_dependencies_1.rxCopyDependencies(pkgFile, dstBase);
    return rxCopy;
}
function _copyDirect(aSrcDir, aDstDir, aLogger) {
    // log this
    aLogger.next("Copying from [" + aSrcDir + "] to [" + aDstDir + "].");
    // copy source files
    return rx_copy_1.rxCopyDir(aSrcDir, aDstDir);
}
function _writeOptions(aTools, aDstDir) {
    var _a;
    // prepare the context
    var data = (_a = {},
        _a[wchtools_1.KEY_API_URL] = aTools.baseUrl,
        _a);
    // write the file
    return rx_walk_1.rxMkdirp(aDstDir).pipe(operators_1.mergeMap(function (dir) { return rx_file_1.writeJsonFileSafe(path_1.join(dir, wchtools_1.FILE_WCHTOOLS_JSON), data, true); }));
}
/**
 * Copies the build output to the correct location
*/
var CopyDataCommand = /** @class */ (function (_super) {
    __extends(CopyDataCommand, _super);
    function CopyDataCommand(options) {
        return _super.call(this, options) || this;
    }
    CopyDataCommand.prototype.printHelp = function () {
        // log the command line
        console.log(CopyDataCommand.COMMAND, CopyDataCommand.TYPE, "--" + constants_1.OPTION_PKG_DIR + " <PKG>", "--" + constants_1.OPTION_AGGREGATED, "--" + constants_1.OPTION_DATA + " <DATA>", "--" + constants_1.OPTION_OUT + " <OUT>", "[--" + constants_1.OPTION_USER + " <USER>]", "[--" + constants_1.OPTION_PASSWORD + " <PWD>]", "[--" + constants_1.OPTION_URL + " <URL>]", "[--" + constants_1.OPTION_OVERRIDE + "]", "[--" + constants_1.OPTION_VERBOSE + "]");
        // log the parameters
        console.log("--" + constants_1.OPTION_PKG_DIR + " <PKG>", ':', help_1.HELP_PKG_DIR);
        console.log("--" + constants_1.OPTION_AGGREGATED, ':', help_1.HELP_AGGREGATED);
        console.log("--" + constants_1.OPTION_OUT + " <OUT>", ':', help_1.HELP_DST_DIR);
        console.log("--" + constants_1.OPTION_DATA + " <DATA>", ':', help_1.HELP_DATA_DIRECTORY);
        console.log("--" + constants_1.OPTION_USER + " <USER>", ':', help_1.HELP_USERNAME);
        console.log("--" + constants_1.OPTION_PASSWORD + " <PWD>", ':', help_1.HELP_PASSWORD);
        console.log("--" + constants_1.OPTION_URL + " <URL>", ':', help_1.HELP_API_URL);
        console.log("--" + constants_1.OPTION_OVERRIDE, ':', help_1.HELP_OVERRIDE);
        console.log("--" + constants_1.OPTION_VERBOSE, ':', help_1.HELP_VERBOSE);
    };
    CopyDataCommand.prototype.process = function () {
        // the options
        var opts = this.getOptions();
        // logger
        var logger = this.getLogger();
        // check if we are aggregated
        var bAggregated = !!opts.aggregated;
        // get the output directory
        var dstDir = options_1.optNormalizeDir(opts.out || 'tmp');
        // depending on this, perform the copy
        var rxCopiedFiles = bAggregated ? _copyAggregated(path_1.join(this.getPackageDir(), 'package.json'), dstDir, logger) : _copyDirect(this.getDataDir(), dstDir, logger);
        // wchtools options
        var rxTools = wchtools_1.wchToolsGetOptions(this.getOptions(), this.getDataDir());
        var rxOptions = rxTools.pipe(operators_1.mergeMap(function (opts) { return _writeOptions(opts, dstDir); }));
        // concatenate
        return rxjs_1.concat(rxCopiedFiles, rxOptions).pipe(operators_1.distinct());
    };
    CopyDataCommand.TYPE = 'data';
    CopyDataCommand.COMMAND = constants_1.CMD_COPY;
    return CopyDataCommand;
}(abstract_command_1.AbstractCommand));
exports.CopyDataCommand = CopyDataCommand;
