"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
exports.__esModule = true;
/* Copyright IBM Corp. 2017 */
var number_to_words_1 = require("number-to-words");
var path_1 = require("path");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var util_1 = require("util");
var uuid_1 = require("uuid");
var layout_mapping_utils_1 = require("../utils/layout.mapping.utils");
var constants_1 = require("./../constants");
var json_1 = require("./../utils/json");
var revision_utils_1 = require("./../utils/revision.utils");
var rx_walk_1 = require("./../utils/rx.walk");
var transliteration_1 = require("./../utils/transliteration");
var wchtools_1 = require("./../utils/wchtools");
var abstract_command_1 = require("./abstract.command");
var TYPE_SUFFIX = 'Type';
var LAYOUT_SUFFIX = 'Layout';
var MAPPING_SUFFIX = 'Mapping';
function _niceStringify(aData) {
    return JSON.stringify(aData, null, 2);
}
function _createWchToolsInfo(aTools) {
    var result = {
        tools: aTools,
        layoutsByName: {},
        mappingByType: {},
        typeByName: {}
    };
    var layoutsByName = result.layoutsByName;
    var mappingByType = result.mappingByType;
    var typeByName = result.typeByName;
    // tslint:disable-next-line:forin
    for (var id in aTools.types) {
        // register
        var type = aTools.types[id];
        typeByName[type.name] = type;
    }
    // tslint:disable-next-line:forin
    for (var id in aTools.layouts) {
        // register
        var layout = aTools.layouts[id];
        layoutsByName[layout.name] = layout;
    }
    // tslint:disable-next-line:forin
    for (var id in aTools.layoutMappings) {
        // register
        var mapping = aTools.layoutMappings[id];
        mappingByType[mapping.type.id] = mapping;
    }
    return result;
}
function _createDefaultLayoutNameMapper(aOptions, aTools) {
    return function (aNumber, aTotal, aTypeName) {
        // if we only have one type
        if (aTotal === 1) {
            // check if we already know a layout for the type
            var type = aTools.typeByName[aTypeName];
            if (type) {
                var typeId = type.id;
                var existingMapping = aTools.mappingByType[typeId];
                if (existingMapping) {
                    // get the default layout from the mapping
                    var defaultLayout = layout_mapping_utils_1.getDefaultLayout(existingMapping, aTools.tools);
                    if (defaultLayout && util_1.isString(defaultLayout.name)) {
                        // returns that layout name
                        return defaultLayout.name;
                    }
                }
            }
        }
        // add the ordinal number
        var prefix = (aTotal > 1) ? number_to_words_1.toWordsOrdinal(aNumber + 1) : '';
        // construct layout name from type name
        var typeName = transliteration_1.camelCase(prefix + ' ' + aTypeName);
        if (typeName.endsWith(TYPE_SUFFIX)) {
            typeName = typeName.substr(0, typeName.length - TYPE_SUFFIX.length);
        }
        // augment
        return typeName + LAYOUT_SUFFIX;
    };
}
function _createLayoutMappingName(aLayoutName) {
    // construct layout name from type name
    return aLayoutName + MAPPING_SUFFIX;
}
function _getUniqueFileNameCount(aCount, aPath, aID) {
    // try if the default name would exist
    var path = path_1.join(aPath, aID + "-" + aCount + ".json");
    return rx_walk_1.rxStats(path).pipe(operators_1.mergeMap(function (stat) { return stat.isFile() ? _getUniqueFileNameCount(aCount + 1, aPath, aID) : rxjs_1.of(path); }), operators_1.catchError(function () { return rxjs_1.of(path); }));
}
function _getUniqueFileName(aPath, aID) {
    // try if the default name would exist
    var path = path_1.join(aPath, aID + ".json");
    return rx_walk_1.rxStats(path).pipe(operators_1.mergeMap(function (stat) { return stat.isFile() ? _getUniqueFileNameCount(0, aPath, aID) : rxjs_1.of(path); }), operators_1.catchError(function () { return rxjs_1.of(path); }));
}
function _createNewLayoutForType(aName, aType, aTools) {
    // construct a new ID
    // const id = v4();
    var id = transliteration_1.kebabCase(aName);
    // check if the ID is already known
    if (aTools.tools.layouts[id]) {
        id = uuid_1.v4();
    }
    // selector
    var selector = transliteration_1.kebabCase(aName);
    // create the path
    var dir = path_1.join(aTools.tools.root, 'layouts');
    var path = rx_walk_1.rxMkdirp(dir);
    // source assets
    var srcThumb = path_1.join(__dirname, '..', '..', '..', 'assets', 'layout.thumbnail.jpg');
    // thumb path
    var relThumPath = '/dxauth/layoutThumbs/';
    var thumbPath = rx_walk_1.rxMkdirp(path_1.join(aTools.tools.root, 'assets', relThumPath));
    // construct the file
    var layout = revision_utils_1.addRevisionId({
        'id': id,
        'name': aName,
        'classification': 'layout',
        'prerender': false,
        'template': selector,
        'templateType': 'angular',
        'thumbnail': {
            'path': "" + relThumPath + id + ".jpg"
        }
    });
    // register this new layout
    aTools.layoutsByName[aName] = layout;
    aTools.tools.layouts[id] = layout;
    // copy the thumbnail
    var thumb = thumbPath.pipe(operators_1.map(function (p) { return path_1.join(p, id + ".jpg"); }), operators_1.mergeMap(function (dst) { return rx_walk_1.rxCopyFile(srcThumb, dst); }));
    // handle
    var json = path.pipe(operators_1.mergeMap(function (p) { return _getUniqueFileName(p, id); }), operators_1.mergeMap(function (name) { return rx_walk_1.writeFile(name, _niceStringify(json_1.canonicalizeJSON(layout))); }));
    // combine
    return rxjs_1.merge(json, thumb);
}
function _createLayoutMapping(aType, aTotal, aNameMapper, aTools) {
    // build the array of layouts
    return rxjs_1.range(0, aTotal).pipe(operators_1.map(function (id) { return aNameMapper(id, aTotal, aType.name); }), operators_1.map(function (name) { return aTools.layoutsByName[name]; }), operators_1.toArray(), operators_1.mergeMap(function (allLayouts) {
        // check if we have a known mapping
        var knownMapping = aTools.mappingByType[aType.id];
        if (!knownMapping) {
            // name of the mapping
            var canonicalLayoutName = aNameMapper(0, 1, aType.name);
            var name_1 = _createLayoutMappingName(canonicalLayoutName);
            // new ID
            var id = transliteration_1.kebabCase(name_1);
            // construct a mapping
            knownMapping = revision_utils_1.addRevisionId({
                'id': id,
                'name': name_1,
                'classification': 'layout-mapping',
                'type': {
                    'id': aType.id,
                    'name': aType.name
                },
                mappings: []
            });
            // the new filename
            aTools.tools.idToFile[id] = path_1.join(aTools.tools.root, 'layout-mappings', id + '.json');
        }
        // filename
        var fileName = aTools.tools.idToFile[knownMapping.id];
        // mappings file
        var mappings = knownMapping.mappings;
        // make sure we have a record
        var defaultRecord = mappings[0] || (mappings[0] = {});
        // check for the default
        var firstLayout = allLayouts[0];
        var defaultLayout = defaultRecord.defaultLayout || (defaultRecord.defaultLayout = {
            'id': firstLayout.id,
            'name': firstLayout.name
        });
        // make sure to register the layout in the list of children
        var layouts = defaultRecord.layouts || (defaultRecord.layouts = []);
        // iterate over all layouts
        allLayouts.forEach(function (aLayout) {
            // do we know our layout
            if (!layouts.find(function (l) { return l.id === aLayout.id; })) {
                // insert our layout
                layouts.push({
                    'id': aLayout.id,
                    'name': aLayout.name
                });
            }
        });
        // serialize, so we can check if we need to update the file
        var data = _niceStringify(json_1.canonicalizeJSON(knownMapping));
        // first read, so we do not overide an unmodified file
        return rx_walk_1.rxMkdirp(path_1.join(aTools.tools.root, 'layout-mappings')).pipe(operators_1.mergeMap(function (dir) { return rx_walk_1.writeFileIfChanged(fileName, data); }));
    }));
}
function _createLayout(aType, aTools, aNumber, aTotal, aNameMapper) {
    // construct a layout name
    var layoutName = aNameMapper(aNumber, aTotal, aType.name);
    // the generated layout
    var layout;
    // check if we know about the layout
    var knownLayout = aTools.layoutsByName[layoutName];
    if (knownLayout) {
        // no layout to create
        layout = rxjs_1.empty();
    }
    else {
        // the layout does not exist
        layout = _createNewLayoutForType(layoutName, aType, aTools);
        knownLayout = aTools.layoutsByName[layoutName];
    }
    // go
    return layout;
}
function _createLayouts(aTools, aTypePredicate, aTotal, aNameMapper, aOptions, aLogger) {
    // some helper
    var tools = _createWchToolsInfo(aTools);
    // construct a mapper
    var nameMapper = aNameMapper || _createDefaultLayoutNameMapper(aOptions, tools);
    // log this
    aLogger.next("Layouts: " + _niceStringify(Object.keys(tools.layoutsByName)));
    aLogger.next("Mappings: " + _niceStringify(Object.keys(tools.mappingByType)));
    aLogger.next("Types: " + _niceStringify(Object.keys(aTools.types)));
    // create a layout for each type
    return rxjs_1.from(Object.keys(aTools.types)).pipe(operators_1.map(function (key) { return aTools.types[key]; }), operators_1.filter(aTypePredicate), operators_1.mergeMap(function (type) { return rxjs_1.concat(rxjs_1.range(0, aTotal).pipe(operators_1.mergeMap(function (idx) { return _createLayout(type, tools, idx, aTotal, nameMapper); })), _createLayoutMapping(type, aTotal, nameMapper, tools)); }));
}
function _allTypes(aType) {
    return true;
}
function _matches(aName, aMatcher, aLogger) {
    // check
    var result = aMatcher(aName);
    aLogger.next("Matching [" + aName + "] results in [" + result + "].");
    return result;
}
function _byMatcher(aMatcher, aLogger) {
    // execute the check
    return function (aType) { return aType && aType.name && _matches(aType.name, aMatcher, aLogger); };
}
/**
 * Generates layout and layout mappings
 *
 * create layouts --data DATA_FOLDER [--count]
 */
var CreateLayoutsCommand = /** @class */ (function (_super) {
    __extends(CreateLayoutsCommand, _super);
    function CreateLayoutsCommand(options) {
        var _this = _super.call(this, options) || this;
        _this.options = options;
        return _this;
    }
    CreateLayoutsCommand.prototype.process = function () {
        var _this = this;
        // defer
        return rxjs_1.defer(function () {
            // logger
            var logger = _this.getLogger();
            // matcher
            var matcher = wchtools_1.wchToolsGetMatcher(_this.options);
            var typeMatcher = _byMatcher(matcher, logger);
            // access the name mapper
            var layoutNameMapper = _this.options.layoutNameMapper;
            // number of layouts to create
            var total = Math.max(1, _this.options.count || 1);
            // the tools
            var rxWchTools = _this.getWchTools();
            // generated layouts
            var rxLayouts = rxWchTools.pipe(operators_1.mergeMap(function (aTools) { return _createLayouts(aTools, typeMatcher, total, layoutNameMapper, _this.options, logger); }));
            // ok
            return rxLayouts;
        });
    };
    CreateLayoutsCommand.TYPE = 'layouts';
    CreateLayoutsCommand.COMMAND = constants_1.CMD_CREATE;
    return CreateLayoutsCommand;
}(abstract_command_1.AbstractCommand));
exports.CreateLayoutsCommand = CreateLayoutsCommand;
