import { Observable } from 'rxjs';
import { mergeMap } from 'rxjs/operators';

import { CMD_SHOW, OPTION_NAME } from './../constants';
import { Options } from './../options';
import { configTrimSlashes } from './../utils/config';
import { wchGetIndexUrl } from './../utils/wchtools';
import { AbstractCommand } from './abstract.command';
import { Command } from './command';
import { HELP_APPLICATION_NAME } from './help';

/* Copyright IBM Corp. 2017 */
export class ShowUrlCommand extends AbstractCommand implements Command {

    static readonly TYPE = 'url';
    static readonly COMMAND = CMD_SHOW;

    constructor(private options: Options) {
        super(options);
    }

    printHelp(): void {
        // log the command line
        console.log(ShowUrlCommand.COMMAND, ShowUrlCommand.TYPE,
            `--${OPTION_NAME} <NAME>`);
        // log the parameters
        console.log(`--${OPTION_NAME} <NAME>`, ':', HELP_APPLICATION_NAME);
    }

    process(): Observable<string> {
        // options
        const opts = this.options;
        // the directories
        const dataDirOpt = this.getDataDir();
        const appName = opts.name || this.getAppName();
        const contextRoot = `/${configTrimSlashes(opts.root || this.getContextRoot(appName))}`;

        return this.getWchToolsOptions().pipe(
            mergeMap(toolsOpt => wchGetIndexUrl(contextRoot, toolsOpt))
        );
    }
}
