"use strict";
exports.__esModule = true;
/* Copyright IBM Corp. 2017 */
var path_1 = require("path");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var rx_copy_1 = require("./rx.copy");
var rx_file_1 = require("./rx.file");
var rx_walk_1 = require("./rx.walk");
var toposort = require('toposort');
exports.KEY_WCHTOOLS_DEPENDENCIES = 'wchtools-dependencies';
exports.DEPENDENCIES = 'dependencies';
function _resolveToPackage(aModule, aRootPath) {
    // check for the existence of a node-modules folder
    var normRoot = path_1.normalize(aRootPath);
    var pkgName = path_1.normalize(path_1.join(normRoot, 'node_modules', aModule, 'package.json'));
    var normParent = path_1.normalize(path_1.join(normRoot, '..'));
    // check if this is a folder
    var rxHasPackageJson = rx_walk_1.rxStats(pkgName).pipe(operators_1.catchError(function () { return rxjs_1.of(undefined); }), operators_1.map(function (stat) { return stat && stat.isFile(); }));
    // test
    return rxHasPackageJson.pipe(operators_1.mergeMap(function (bExists) { return bExists
        ? rxjs_1.of(pkgName) : (normRoot === normParent)
        ? rxjs_1.throwError(aModule) : _resolveToPackage(aModule, normParent); }));
}
function _isRelativePath(aPath) {
    return aPath.startsWith('./') || aPath.startsWith('../');
}
function _resolveEdges(aPackageFile, aCycle) {
    // load the package file
    var rxPackage = rx_file_1.rxReadTextFile(aPackageFile).pipe(operators_1.map(function (data) { return JSON.parse(data); }));
    // extract the dependencies
    var rxDeps = rxPackage.pipe(operators_1.mergeMap(function (pkg) { return rxjs_1.from((pkg[exports.KEY_WCHTOOLS_DEPENDENCIES] || Object.keys(pkg[exports.DEPENDENCIES] || {}))); }));
    // work on the dependencies and resolve the modules
    return rxDeps.pipe(operators_1.mergeMap(function (dep) {
        // root path
        var parsed = path_1.parse(aPackageFile);
        // handle relative paths
        if (_isRelativePath(dep)) {
            // just resolve
            return rxjs_1.of([aPackageFile, path_1.normalize(path_1.join(parsed.dir, dep))]);
        }
        // check of a cycle on the module name
        var rxCycle = aCycle[dep];
        if (rxCycle) {
            // break cycle
            return rxCycle.pipe(operators_1.map(function (pkg) { return [aPackageFile, pkg]; }));
        }
        // register the module
        rxCycle = aCycle[dep] = _resolveToPackage(dep, parsed.dir).pipe(operators_1.shareReplay(1));
        // resolve dependency
        return rxCycle.pipe(operators_1.mergeMap(function (pkg) { return _resolveEdges(pkg, aCycle).pipe(operators_1.startWith([aPackageFile, pkg])); }));
    }));
}
function _isPackageJson(aPath) {
    var parsed = path_1.parse(aPath);
    return parsed.base === 'package.json';
}
function rxResolveEdges(aPackageFile) {
    return _resolveEdges(aPackageFile, {});
}
exports.rxResolveEdges = rxResolveEdges;
/**
 * Returns the 'wchtools-dependencies' in topological order
 *
 * @param aPackageFile  the package file
 * @return the dependencies in topological order
 */
function rxResolveDependencies(aPackageFile, aReversedOrder) {
    return rxResolveEdges(aPackageFile).pipe(operators_1.toArray(), operators_1.map(toposort), operators_1.map(function (res) { return aReversedOrder ? res : res.reverse(); }), operators_1.mergeMap(function (res) { return rxjs_1.from(res); }), operators_1.filter(function (res) { return !_isPackageJson(res); }));
}
exports.rxResolveDependencies = rxResolveDependencies;
function _distinctKey(aDesc) {
    return aDesc.relPath;
}
/**
 * Provides a listing of all files to copie copied, in overlayed order. The relative
 * paths of the result are the relevant target paths
 *
 * @param aPackageFile  the package file
 * @return observable of the copied files
 */
function rxAggregateDependencies(aPackageFile) {
    // walk over the dependencies
    return rxResolveDependencies(aPackageFile, true).pipe(operators_1.concatMap(rx_walk_1.rxWalkFiles), operators_1.distinct(_distinctKey));
}
exports.rxAggregateDependencies = rxAggregateDependencies;
/**
 * Provides a listing of all files to copie copied, in overlayed order. The relative
 * paths of the result are the relevant target paths
 *
 * @param aPackageFile  the package file
 * @param aDstDir target directory
 * @return observable of the copied files
 */
function rxCopyDependencies(aPackageFile, aDstDir) {
    // walk over the dependencies
    return rx_copy_1.rxCopyFiles(rxAggregateDependencies(aPackageFile), aDstDir);
}
exports.rxCopyDependencies = rxCopyDependencies;
